from setuptools import setup, find_packages

setup(
    name='pyavlss2025',
    version='0.1.2',
    description='LSS-2025',
    author='Alessandro Vaccari',
    author_email='alessandro.vaccari@unife.it',
    packages=find_packages(),
    python_requires='>=3.6',
)